/**
 * Created by Thaylin Burns ©2009 for Flash Coders NY http://flashcodersny.org
 * Visit my blog for more actionscript tips at http://thaylin.info/blog 
 * 
 * For a 6 page article on this and a little on injecting VBScript into a page via as3 visit the following link:
 * http://www.actionscript.org/resources/articles/745/1/JavaScript-and-VBScript-Injection-in-ActionScript-3/Page1.html
 * 
*/
package org.flashCodersNY.thaylin
{
	import flash.external.ExternalInterface;
	
	/**
	 * 
	 * NOTES:
	 * ExternalInterface is a replacement for fscommand()
	 * 
	 * Taken from the as3 documentation:
	 * From ActionScript, you can call a JavaScript function on the HTML page. The external API offers the following improved functionality compared with fscommand():
	 * You can use any JavaScript function, not only the functions that you can use with the fscommand() function.
	 * You can pass any number of arguments, with any names; you aren't limited to passing a command and a single string argument. This gives the external API much more flexibility than fscommand().
	 * You can pass various data types (such as Boolean, Number, and String); you are no longer limited to String parameters.
	 * You can receive the value of a call, and that value returns immediately to ActionScript (as the return value of the call you make).
	 * 
	 * IMPORTANT:
	 * In addition, if the HTML tags that define the Flash Player instance (the object and embed tags) are nested in
	 * an HTML form tag, ExternalInterface calls from ActionScript will not work.
	 * 
	 * ALSO IMPORTANT:
	 * If the name given to the Flash Player instance in an HTML page (the object tag's id attribute) includes a hyphen (-)
	 * or other characters that are defined as operators in JavaScript (such as +, *, /, \, ., and so on), ExternalInterface calls
	 * from ActionScript will not work when the container web page is viewed in Internet Explorer.
	 * 
	 */	
	public class Example1_ExternalInterface
	{
		
		/**
		 *IN ORDER TO TEST THIS THE PROJECT MUST BE SET UP TO RUN ON A SERVER. GO TO PROJECT PROPERTIES>BUILD PATH> 
		 * 
		 */		
		public function Example1_ExternalInterface()
		{
		}
		public function start():void
		{
			/**
			 *CHECK FOR AVAILABILITY 
			 */			
			
			/* try{
				ExternalInterface.call("alert", "hi there")
			}catch(e:Error){
				trace('EXTERNAL INTERFACE UNAVAILABLE')
			}  */
			
			/**
			 * BASIC EXTERNAL INTERFACE CALL 
			 * FLASH PASSES THIS TO THE BROWSER AS A STRING WHICH IN TURN USES JAVASCRIPT EVAL() TO CONVERT TO JS CODE
			 */			
			//ExternalInterface.call("alert('hi there')");
			
			/**
			 *USING BASIC VARS 
			 */			
			var jsFunc:String = 'alert';
			var jsVar:String = 'oh hellooooooo';
			//ExternalInterface.call(jsFunc, jsVar)

			/**
			 * CALLING JS VIA BASIC XML
			 * BELOW WE'VE ALSO ADDED AN OBJECT THAT YOU CAN PASS TO FLASH BY RETURNING YOUR OBJECT IN THE METHOD AT WHICH POINT FLASH RETURNS THAT IN THE EXTERNALINTERFACE.CALL() METHOD
			 */			
			var jsXMLnoVar:XML = 
			<script>
				<![CDATA[
							function ()
							{
								var message = 'Hi there Im an alert'
								alert(message);
								FCNY = new Object();
								bool = false;
								myStr = "Oh hello there I\'m a string"
								return {object:FCNY, boolean:bool, myStr:myStr};
							}
				]]>
			</script>
			
			var myObj:Object = ExternalInterface.call(jsXMLnoVar);
			// YOU CAN PLACE A BREAK POINT AT TRACE POINT TO SEE THE OBJECT THAT'S BEEN PASSED
			trace(myObj)
			/**
			*
			*/
			
			
			/**
			 *WE CAN EVEN PASS VARIABLES TO THE JAVASCRIPT 
			 */			
			var jsXMLvar:XML = 
			<script>
				<![CDATA[
							function (message)
							{
								alert(message)
							}
				]]>
			</script>
			//ExternalInterface.call(jsXMLvar, 'hi there I am passing varz')
			
			/**
			 *HERE WE SEE WE CAN PASS MULTIPLE VARIABLES 
			 */			
			var jsXMLvars2:XML = 
			<script>
				<![CDATA[
							function(val, val2)
							{				
								function callAlert(message, message2)
								{
									alert(message)
									callSecondAlert(message2)
								}
								function callSecondAlert(message)
								{
									alert(message)
								}
								callAlert(val, val2)
								
							}
				]]>
			</script>
			//ExternalInterface.call(jsXMLvars2, 'first var', 'second var');
			
			/**
			* ALL OF THE PREVIOUS CALLS THAT ARE PASSED AREN'T SAVED IN MEMORY.
			* ONCE THEY'RE FIRED THEY'RE FORGOTTEN AND REMOVED FROM THE RAM
			* 
			* BUT WHAT IF WE WANT SOMETHING TO PERSIST GLOBALLY IN THE BROWSER?
			* 
			*/	
			var persistJsXML:XML = 
			<script>
				<![CDATA[
							function()
							{
								//FOR PERSISTENT VARIABLES JUST REMOVE THE 'var' ATTRIBUTE FROM THE CREATION OF THE OBJECT
								var myNonPersistentVar = 'FCNY!';
								var nonPersistentFunction = function()
								{
									alert('non persistent function')	
								}
								//BUT THIS ISN'T WISE AS THERE COULD BE OBJECTS WITH YOUR OBJECTS NAME ALREADY IN MEMORY
								//SO INSTEAD LETS CREATE A NAMESPACE TO STORE YOUR VARIABLES
								//FIRST WE NEED TO CHECK FOR THIS NAMESPACE TO SEE IF IT EXISTS
								if(window.FCNY)
								{
									alert('FCNY EXISTS')
									FCNY.myPersistentVar = 'FCNY!'
									FCNY.myPersistentFunction = function (value)
									{
										alert('Passed to FCNY.myPersistentFunction '+ value);
										alert('also referencing the nonPersistentVar to keep in memory in this wrapper function '+myNonPersistentVar)
									}
									FCNY.callNonPersistentFunction = function()
									{
										nonPersistentFunction();
									}
								}else
								{
									alert('FCNY DOESNT EXIST')	
								}
							}
				]]>
			</script>
			var createFCNYObject:XML =
			<script>
				<![CDATA[
							function()
							{
								if(!window.FCNY)
								{
									FCNY = new Object();
								}
							}
				]]>
			</script>
			//
			// YOU CAN COMMENT AND UNCOMMENT THESE CALLS IN VARIOUS WAYS TO SEE THE IMPACT EACH DOES AND HOW ACH WORKS WITH THE OTHER.
			//
			// FIRST WE CREATE OUR FCNY OBJECT
			//ExternalInterface.call(createFCNYObject);
			
			// THEN WE CALL OUR XML WHICH WILL RUN THE MAIN FUNCTION AND IF THE PREVIOUS CALL WAS MADE IT WILL SEE THAT THE WINDOW.FCNY OBJECT EXISTS
			// AT THIS POINT IT WILL THROW AN ALERT AND CREATE PERSISTENT VARIABLES AND METHODS AS WELL AS A NON PERSISTENT METHOD
			//ExternalInterface.call(persistJsXML);
			
			// THIS CALLS THE PERSISTENT FUNCTION WHITHIN THE FCNY OBJECT AND PASSES THE VALUE 'TEST' TO IT SHOWING THAT IT INDEEDS EXISTS AFTER ITS BEEN CREATED AND THEN CALLED SEPERATELY
			//ExternalInterface.call('FCNY.myPersistentFunction', 'test ');
			
			// THIS METHOD IS CALLED TO SHOW THAT A PERSISTENT METHOD CAN BECOME A WRAPPER TO ACCESS NON PERSISTENT METHODS
			//ExternalInterface.call('FCNY.callNonPersistentFunction');
			
			// AND FINALLY THIS METHOD SHOWS THAT THE NONPERSISTENT METHOD INDEED DOES NOT EXIST AS THE PERSISTENT METHOD DOES AND CANNOT BE CALLED SEPERATELY FROM IT'S INSTANTIATION
			//ExternalInterface.call('nonPersistentFunction');
			
		}		
	}
}